package com.example.demo.orders.model;

import com.example.demo.core.model.BaseEntity;
import com.example.demo.order_lines.model.OrderLineEntity;
import com.example.demo.users.model.UserEntity;

import jakarta.persistence.CascadeType;
import jakarta.persistence.Entity;
import jakarta.persistence.JoinColumn;
import jakarta.persistence.ManyToMany;
import jakarta.persistence.ManyToOne;
import jakarta.persistence.Table;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Entity
@Table(name = "orders")
public class OrderEntity extends BaseEntity {
    @ManyToOne
    @JoinColumn(name = "userId", nullable = false)
    private UserEntity user;
    @ManyToMany(cascade = CascadeType.ALL)
    private final List<OrderLineEntity> lines = new ArrayList<>();

    public OrderEntity() {
        super();
    }

    public OrderEntity(UserEntity user) {
        this.user = user;
    }

    public UserEntity getUser() {
        return user;
    }

    public void setUser(UserEntity user) {
        this.user = user;
    }

    public List<OrderLineEntity> getLines() {
        return lines;
    }

    public void addOrderLine(OrderLineEntity orderLine) {
        this.lines.add(orderLine);
    }

    public void removeOrderLine(OrderLineEntity orderLine) {
        this.lines.remove(orderLine);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, user, lines);
    }

    @SuppressWarnings("unlikely-arg-user")
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null || getClass() != obj.getClass())
            return false;
        final OrderEntity other = (OrderEntity) obj;
        return Objects.equals(other.getId(), id)
                && Objects.equals(other.getUser(), user)
                && Objects.equals(other.getLines(), lines);
    }

    public Double calculateTotalOrderPrice() {
        Double totalPrice = 0.0;
        for (OrderLineEntity orderLine : lines) {
            totalPrice += orderLine.getProduct().getPrice() * orderLine.getCount();
        }
        return totalPrice;
    }
}
